abstract public class LLNode{
  /**
   * The next branch/node/whatever to process
   **/
  public LLNode next;
  
  /**
   * Constructor for LLNode just sets
   * next to null
   **/
  public LLNode(){
    next = null;
  }
  
  /**
   * Methods to set and get next elements
   * @param nextOne next element in list
   **/
  public void setNext(LLNode nextOne){
    this.next = nextOne;
  }
  
  public LLNode getNext(){
    return this.next;
  }
  
  
  /** Method to remove node from list, fixing links appropriately.
   * @param node element to remove from list.
   **/
  public void remove(LLNode node){
    if (node==this)
    {
      System.out.println("I can't remove the first node from the list.");
      return;
    };
    
    LLNode current = this;
    // While there are more nodes to consider
    while (current.getNext() != null)
    {
      if (current.getNext() == node){
        // Simply make node's next be this next
        current.setNext(node.getNext());
        // Make this node point to nothing
        node.setNext(null);
        return;
      }
      current = current.getNext();
    }
  }

    
  /**
   * Insert the input node after this node.
   * @param node element to insert after this.
   **/
  public void insertAfter(LLNode node){
    // Save what "this" currently points at
    LLNode oldNext = this.getNext();
    this.setNext(node);
    node.setNext(oldNext);
  }
  
  /**
   * Return the last element in the list
   **/
    public LLNode last() {
      LLNode current;
    
      current = this;
      while (current.getNext() != null)
      {
        current = current.getNext();
      };
      return current;
  }
  /**
   * Return the count of the elements in the list
   **/
    public int count() {
      LLNode current;
      int count = 1;
    
      current = this;
      while (current.getNext() != null)
      {
        count++;
        current = current.getNext();
      };
      return count;
  }


  /**
   * Add the input node after the last node in this list.
   * @param node element to insert after this.
   **/
  public void add(LLNode node){
    this.last().insertAfter(node);
  }

  /**
   * Reverse the list starting at this, 
   * and return the last element of the list.
   * The last element becomes the FIRST element
   * of the list, and THIS goes to null.
   **/
    public LLNode reverse() {
      LLNode reversed, temp;
    
    // Handle the first node outside the loop
    reversed = this.last();
    this.remove(reversed);
    
    while (this.getNext() != null)
    {
      temp = this.last();
      this.remove(temp);
      reversed.add(temp);
    };
    
    // Now put the head of the old list on the end of
    // the reversed list.
    reversed.add(this);
    
    // At this point, reversed 
    // is the head of the list
    return reversed;
  }
    
   /**
   * Reverse2: Push all the elements on
   * the stack, then pop all the elements 
   * off the stack.
   **/
    public LLNode reverse2() {
      LLNode reversed, current, popped;
      Stack stack = new Stack();
      
      current=this;
      while (current != null)
      {
        stack.push(current);
        current = current.getNext();
      }
    
      reversed = (LLNode) stack.pop();
      current = reversed;
      while (stack.size()>0) {
        popped=(LLNode) stack.pop();
        current.insertAfter(popped);
        current = popped;
      }
      
      return reversed;
    }
    
}

